-- CREATE DATABASE electrodomestics

----------------------------------------------------------------------
-- Neteja de taules
----------------------------------------------------------------------
DROP TABLE IF EXISTS adquisicio;
DROP TABLE IF EXISTS comunicacio;
DROP TABLE IF EXISTS especialitzacio;

DROP TABLE IF EXISTS accio;
DROP TABLE IF EXISTS incidencia;
DROP TABLE IF EXISTS tipus_inc;

DROP TABLE IF EXISTS producte;
DROP TABLE IF EXISTS tipus_prd;
DROP TABLE IF EXISTS gamma;

DROP TABLE IF EXISTS client;
DROP TABLE IF EXISTS factura;

DROP TABLE IF EXISTS venedor;
DROP TABLE IF EXISTS agent_telefonic;
DROP TABLE IF EXISTS enginyer;
DROP TABLE IF EXISTS empleat;

----------------------------------------------------------------------
-- TAULA gamma -- Gammes d'electrodomestics que es fabricaran
----------------------------------------------------------------------
CREATE TABLE gamma (
  codiGamma  CHAR(1)     CONSTRAINT PK_gamma     PRIMARY KEY,
  descripcio VARCHAR(16) CONSTRAINT NN_gamma_dsc NOT NULL,
                         CONSTRAINT UN_gamma_dsc UNIQUE (descripcio)
);

----------------------------------------------------------------------
-- TAULA tipus_prd -- Tipus de productes que es fabricaran
----------------------------------------------------------------------
CREATE TABLE tipus_prd (
  codiTipusPrd CHAR(4)     CONSTRAINT PK_tipus_prd     PRIMARY KEY,
  gamma        CHAR(1)     CONSTRAINT FK_tipus_prd_gam
                                      REFERENCES gamma(codiGamma)
                           CONSTRAINT NN_tipus_prd_gam NOT NULL,
  descripcio   VARCHAR(32) CONSTRAINT NN_tipus_prd_dsc NOT NULL,
                           CONSTRAINT UN_tipus_prd_dsc UNIQUE (descripcio)
);

----------------------------------------------------------------------
-- TAULA producte -- Productes
----------------------------------------------------------------------
CREATE TABLE producte (
  codiProducte   CHAR(12)    CONSTRAINT PK_producte     PRIMARY KEY,
  descripcio     VARCHAR(32) CONSTRAINT NN_producte_dsc NOT NULL,
  tipus          CHAR(4)     CONSTRAINT FK_producte_tip
                                        REFERENCES tipus_prd(codiTipusPrd)
                             CONSTRAINT NN_producte_tip NOT NULL,
  preuCost       NUMERIC     CONSTRAINT NN_producte_prc NOT NULL
                             CHECK (preuCost > 0),
  preuVenda      NUMERIC     CONSTRAINT NN_producte_prv NOT NULL
                             CONSTRAINT CH_producte_prv CHECK (preuVenda > 0),
  duradaGarantia INTEGER     CONSTRAINT NN_producte_dGr NOT NULL
);

----------------------------------------------------------------------
-- TAULA client -- Clients, empreses que ens compren els productes
----------------------------------------------------------------------
CREATE TABLE client (
  codiClient CHAR(8)     CONSTRAINT PK_client     PRIMARY KEY,
  nif        CHAR(9)     CONSTRAINT NN_client_nif NOT NULL,
  nom        VARCHAR(42) CONSTRAINT NN_client_nom NOT NULL,
  adress     VARCHAR(60) CONSTRAINT NN_client_adr NOT NULL,
  telefon    CHAR(9)     CONSTRAINT NN_client_tel NOT NULL,
  descompte  NUMERIC     DEFAULT 0.0
                         CONSTRAINT NN_client_des NOT NULL,
                         CONSTRAINT CH_client_des CHECK (descompte >= 0),
                         CONSTRAINT UN_client_nif UNIQUE (nif)
);

----------------------------------------------------------------------
-- TAULA empleat -- Empleats de l'empresa
----------------------------------------------------------------------
CREATE TABLE empleat (
  codiEmpleat CHAR(4)     CONSTRAINT PK_empleat     PRIMARY KEY,
  nif         CHAR(9)     CONSTRAINT NN_empleat_nif NOT NULL,
  nom         VARCHAR(42) CONSTRAINT NN_empleat_nom NOT NULL,
  adress      VARCHAR(60) CONSTRAINT NN_empleat_adr NOT NULL,
  telefon     CHAR(9),
  nss         CHAR(14)    CONSTRAINT NN_empleat_nss NOT NULL,
                          CONSTRAINT UN_empleat_nss UNIQUE (nss),
                          CONSTRAINT UN_empleat_nif UNIQUE (nif)
);

----------------------------------------------------------------------
-- TAULA venedor -- Venedors, subtipus d'empleats
----------------------------------------------------------------------
CREATE TABLE venedor (
  codiEmpleat    CHAR(4) CONSTRAINT PK_venedor     PRIMARY KEY
                         CONSTRAINT FK_venedor_cod
                                    REFERENCES empleat(codiEmpleat),
  comissioVendes NUMERIC CONSTRAINT NN_venedor_com NOT NULL
);

----------------------------------------------------------------------
-- TAULA agent_telefonic -- Agents telefonics, subtipus d'empleats
----------------------------------------------------------------------
CREATE TABLE agent_telefonic (
  codiEmpleat        CHAR(4) CONSTRAINT PK_agent_telefonic     PRIMARY KEY
                             CONSTRAINT FK_agent_telefonic_cod
                                        REFERENCES empleat(codiEmpleat),
  telefonDepartament CHAR(9) CONSTRAINT NN_agent_telefonic_tlf NOT NULL
);

----------------------------------------------------------------------
-- TAULA enginyer -- Enginyers, subtipus d'empleats
----------------------------------------------------------------------
CREATE TABLE enginyer (
  codiEmpleat  CHAR(4)     CONSTRAINT PK_enginyer     PRIMARY KEY
                           CONSTRAINT FK_enginyer_cod REFERENCES empleat(codiEmpleat),
  especialitat VARCHAR(60) CONSTRAINT NN_enginyer_tlf NOT NULL
);

----------------------------------------------------------------------
-- TAULA especialitzacio -- Relacio entre enginyer i gammes en que estan especcialitzats
----------------------------------------------------------------------
CREATE TABLE especialitzacio (
  enginyer CHAR(4) CONSTRAINT FK_especialitzacio_eng REFERENCES enginyer(codiEmpleat)
                   CONSTRAINT NN_especialitzacio_eng NOT NULL,
  gamma    CHAR(1) CONSTRAINT FK_especialitzacio_gam REFERENCES gamma(codiGamma)
                   CONSTRAINT NN_especialitzacio_gam NOT NULL,
                   CONSTRAINT PK_especialitzacio     PRIMARY KEY(enginyer, gamma)
);

----------------------------------------------------------------------
-- TAULA factura -- Factures emeses per l'empresa que certifiquen una venda
----------------------------------------------------------------------
CREATE TABLE factura (
  codiFactura CHAR(10) CONSTRAINT PK_factura_num PRIMARY KEY,
  venedor     CHAR(4)  CONSTRAINT FK_factura_ven REFERENCES venedor(codiEmpleat)
                       CONSTRAINT NN_factura_ven NOT NULL,
  data        DATE     CONSTRAINT NN_factura_dat NOT NULL
);

----------------------------------------------------------------------
-- TAULA adquisicio -- Relacio entre factura, client i producte
----------------------------------------------------------------------
CREATE TABLE adquisicio (
  producte CHAR(12) CONSTRAINT FK_adquisicio_prd REFERENCES producte(codiProducte)
                    CONSTRAINT NN_adquisicio_prd NOT NULL,
  factura  CHAR(10) CONSTRAINT FK_adquisicio_fct REFERENCES factura(codiFactura)
                    CONSTRAINT NN_adquisicio_fct NOT NULL,
  client   CHAR(8)  CONSTRAINT FK_adquisicio_clt REFERENCES client(codiClient)
                    CONSTRAINT NN_adquisicio_clt NOT NULL,
                    CONSTRAINT PK_adquisicio     PRIMARY KEY(producte, factura, client),
                    CONSTRAINT UN_adquisicio     UNIQUE (producte)
);

----------------------------------------------------------------------
-- TAULA tipus_inc -- Codificacio dels tipus d'incidencies
----------------------------------------------------------------------
CREATE TABLE tipus_inc (
  codiTipusInc      CHAR(4)     CONSTRAINT PK_tipus_inc     PRIMARY KEY,
  descripcio        VARCHAR(32) CONSTRAINT NN_tipus_inc_dsc NOT NULL,
  garantiaAplicable CHAR(1)     CONSTRAINT NN_tipus_inc_gar NOT NULL,
                                CONSTRAINT CH_tipus_inc_gar
                                     CHECK(garantiaAplicable IN ('S', 'N'))
);

----------------------------------------------------------------------
-- TAULA incidencia -- Incidencies postvenda
----------------------------------------------------------------------
CREATE TABLE incidencia (
  codiIncidencia  CHAR(12)    CONSTRAINT PK_incidencia     PRIMARY KEY,
  descripcio      VARCHAR(80) CONSTRAINT NN_incidencia_dsc NOT NULL,
  producte        CHAR(12)    CONSTRAINT FK_incidencia_prd REFERENCES producte(codiProducte)
                              CONSTRAINT NN_incidencia_prd NOT NULL,
  factura         CHAR(10)    CONSTRAINT FK_incidencia_fct REFERENCES factura(codiFactura)
                              CONSTRAINT NN_incidencia_fct NOT NULL,
  tipusIncidencia CHAR(4)     CONSTRAINT FK_incidencia_tip REFERENCES tipus_inc(codiTipusInc)
                              CONSTRAINT NN_incidencia_tip NOT NULL,
  dataObertura    DATE        CONSTRAINT NN_incidencia_dtO NOT NULL,
  dataTancament   DATE,
                              CONSTRAINT CH_incidencia_dtT
                                         CHECK (dataTancament >= dataObertura)
);
--
----------------------------------------------------------------------
-- TAULA comunicacio -- Relacio de comunicacio d'una incidencia
----------------------------------------------------------------------
CREATE TABLE comunicacio (
  incidencia     CHAR(12) CONSTRAINT FK_comunicacio_inc
                                     REFERENCES incidencia(codiIncidencia)
                          CONSTRAINT NN_comunicacio_inc NOT NULL,
  client         CHAR(8)  CONSTRAINT FK_comunicacio_clt
                                     REFERENCES client(codiClient)
                          CONSTRAINT NN_comunicacio_clt NOT NULL,
  agentTelefonic CHAR(4)  CONSTRAINT FK_comunicacio_atl
                                     REFERENCES agent_telefonic(codiEmpleat)
                          CONSTRAINT NN_comunicacio_atl NOT NULL,
                          CONSTRAINT PK_comunicacio
                                     PRIMARY KEY(incidencia, client, agentTelefonic)
);
--
----------------------------------------------------------------------
-- TAULA accio -- Accions que es realitzaran en una incidencia
----------------------------------------------------------------------
CREATE TABLE accio (
  codiAccio  CHAR(12)    CONSTRAINT PK_accio     PRIMARY KEY,
  incidencia CHAR(12)    CONSTRAINT FK_accio_inc REFERENCES incidencia(codiIncidencia)
                         CONSTRAINT NN_accio_inc NOT NULL,
  enginyer   CHAR(4)     CONSTRAINT FK_accio_eng REFERENCES enginyer(codiEmpleat)
                         CONSTRAINT NN_accio_eng NOT NULL,
  descripcio VARCHAR(80) CONSTRAINT NN_accio_dsc NOT NULL,
  dataInici  DATE        CONSTRAINT NN_accio_dtI NOT NULL,
  dataFi     DATE        CONSTRAINT CH_accio_dtF CHECK (dataFi >= dataInici),
  cost       NUMERIC     DEFAULT 0.0
                         CONSTRAINT CH_accio_cst CHECK (cost >= 0)
);

